import { FirestoreAction, FirestoreCommandOptions, FixtureData } from './buildFirestoreCommand';
import { RTDBAction, RTDBCommandOptions } from './buildRtdbCommand';
export interface AttachCustomCommandParams {
    Cypress: any;
    cy: any;
    firebase: any;
}
declare global {
    namespace Cypress {
        interface Chainable {
            /**
             * Login to Firebase auth using FIREBASE_AUTH_JWT environment variable and TEST_UID
             * environment variables.
             * @see https://github.com/prescottprue/cypress-firebase#cylogin
             * @example
             * cy.login()
             */
            login: () => Chainable;
            /**
             * Log out of Firebase instance
             * @see https://github.com/prescottprue/cypress-firebase#cylogout
             * @example
             * cy.logout()
             */
            logout: () => Chainable;
            /**
             * Call Real Time Database path with some specified action. Authentication is through
             * `FIREBASE_TOKEN` (CI token) since firebase-tools is used under the hood, allowing
             * for adming privileges.
             * @param action - The action type to call with (set, push, update, remove)
             * @param actionPath - Path within RTDB that action should be applied
             * @param opts - Options
             * @param opts.args - Command line args to be passed
             * @see https://github.com/prescottprue/cypress-firebase#cycallrtdb
             * @example <caption>Set Data</caption>
             * const fakeProject = { some: 'data' }
             * cy.callRtdb('set', 'projects/ABC123', fakeProject)
             * @example <caption>Set Data With Meta Data</caption>
             * const fakeProject = { some: 'data' }
             * // Adds createdAt and createdBy (current user's uid) on data
             * cy.callRtdb('set', 'projects/ABC123', fakeProject, { withMeta: true })
             * @example <caption>Passing Other Arguments</caption>
             * const options = { args: ['-d'] }
             * const fakeProject = { some: 'data' }
             * cy.callRtdb('update', 'project/test-project', fakeProject, options)
             */
            callRtdb: (action: RTDBAction, actionPath: string, fixtureDataOrPath?: FixtureData | string, opts?: RTDBCommandOptions) => Chainable;
            /**
             * Call Firestore instance with some specified action. Authentication is through
             * serviceAccount.json since it is at the base
             * level. If using delete, auth is through `FIREBASE_TOKEN` since
             * firebase-tools is used (instead of firebaseExtra).
             * @param action - The action type to call with (set, push, update, remove)
             * @param actionPath - Path within RTDB that action should be applied
             * @param opts - Options
             * @param opts.args - Command line args to be passed
             * @see https://github.com/prescottprue/cypress-firebase#cycallfirestore
             * @example <caption>Set Data</caption>
             * const project = { some: 'data' }
             * cy.callFirestore('set', 'project/test-project', project)
             * @example <caption>Add New Document</caption>
             * const project = { some: 'data' }
             * cy.callFirestore('add', 'projects', project)
             * @example <caption>Basic Get</caption>
             * cy.callFirestore('get', 'projects/test-project').then((project) => {
             *   cy.log('Project:', project)
             * })
             * @example <caption>Recursive Delete</caption>
             * const opts = { recursive: true }
             * cy.callFirestore('delete', 'project/test-project', opts)
             * @example <caption>Manually Loading Fixture</caption>
             * cy.fixture('fakeProject.json').then((project) => {
             *   cy.callFirestore('add', 'projects', project)
             * })
             * @example <caption>Fixture Path</caption>
             * cy.callFirestore('set', 'project/test-project', 'fakeProject.json')
             * @example <caption>Other Args</caption>
             * const opts = { args: ['-r'] }
             * cy.callFirestore('delete', 'project/test-project', opts)
             */
            callFirestore: (action: FirestoreAction, actionPath: string, fixtureDataOrPath?: FixtureData | string, opts?: FirestoreCommandOptions) => Chainable;
        }
    }
}
/**
 * Attach custom commands including cy.login, cy.logout, cy.callRtdb,
 * @param commandParams - List of params to provide scope during
 * custom command attachment
 */
export default function attachCustomCommands(commandParams: AttachCustomCommandParams): void;
