import { __assign } from "tslib";
import { isObject } from 'lodash';
import buildFirestoreCommand from './buildFirestoreCommand';
import buildRtdbCommand from './buildRtdbCommand';
/**
 * Attach custom commands including cy.login, cy.logout, cy.callRtdb,
 * @param commandParams - List of params to provide scope during
 * custom command attachment
 */
export default function attachCustomCommands(commandParams) {
    var Cypress = commandParams.Cypress, cy = commandParams.cy, firebase = commandParams.firebase;
    /**
     * Login to Firebase auth using FIREBASE_AUTH_JWT environment variable
     * which is generated using firebase-admin authenticated with serviceAccount
     * during test:buildConfig phase.
     * @name cy.login
     */
    Cypress.Commands.add('login', function () {
        /** Log in using token * */
        if (!Cypress.env('FIREBASE_AUTH_JWT')) {
            cy.log('FIREBASE_AUTH_JWT must be set to cypress environment in order to login');
        }
        else if (firebase.auth().currentUser) {
            cy.log('Authed user already exists, login complete.');
        }
        else {
            return new Promise(function (resolve, reject) {
                firebase.auth().onAuthStateChanged(function (auth) {
                    if (auth) {
                        resolve(auth);
                    }
                });
                firebase
                    .auth()
                    .signInWithCustomToken(Cypress.env('FIREBASE_AUTH_JWT'))
                    .catch(reject);
            });
        }
    });
    /**
     * Log out of Firebase instance
     * @name cy.logout
     * @see https://github.com/prescottprue/cypress-firebase#cylogout
     * @example
     * cy.logout()
     */
    Cypress.Commands.add('logout', function () {
        return new Promise(function (resolve, reject) {
            firebase.auth().onAuthStateChanged(function (auth) {
                if (!auth) {
                    resolve();
                }
            });
            firebase
                .auth()
                .signOut()
                .catch(reject);
        });
    });
    /**
     * Call Real Time Database path with some specified action. Authentication is through
     * FIREBASE_TOKEN since firebase-tools is used (instead of firebaseExtra).
     * @param action - The action type to call with (set, push, update, remove)
     * @param actionPath - Path within RTDB that action should be applied
     * @param opts - Options
     * @param opts.args - Command line args to be passed
     * @name cy.callRtdb
     */
    Cypress.Commands.add('callRtdb', function (action, actionPath, data, opts) {
        // If data is an object, create a copy to original object is not modified
        var dataToWrite = isObject(data) ? __assign({}, data) : data;
        // Add metadata to dataToWrite if specified by options
        if (isObject(data) && opts && opts.withMeta) {
            dataToWrite.createdBy = Cypress.env('TEST_UID');
            dataToWrite.createdAt = firebase.database.ServerValue.TIMESTAMP;
        }
        // Build command to pass to firebase-tools-extra
        var rtdbCommand = buildRtdbCommand(Cypress, action, actionPath, dataToWrite, opts);
        cy.log("Calling RTDB command:\n" + rtdbCommand);
        // Call firebase-tools-extra command
        return cy.exec(rtdbCommand).then(function (out) {
            var stdout = out.stdout, stderr = out.stderr;
            // Reject with Error if error in rtdbCommand call
            if (stderr) {
                cy.log("Error in Firestore Command:\n" + stderr);
                return Promise.reject(stderr);
            }
            // Parse result if using get action so that data can be verified
            if (action === 'get' && typeof stdout === 'string') {
                try {
                    return JSON.parse(stdout);
                }
                catch (err) {
                    cy.log('Error parsing data from callRtdb response', out);
                    return Promise.reject(err);
                }
            }
            // Otherwise return unparsed output
            return stdout;
        });
    });
    /**
     * Call Firestore instance with some specified action. Authentication is through serviceAccount.json since it is at the base
     * level. If using delete, auth is through `FIREBASE_TOKEN` since firebase-tools is used (instead of firebaseExtra).
     * @param action - The action type to call with (set, push, update, remove)
     * @param actionPath - Path within RTDB that action should be applied
     * @param opts - Options
     * @param opts.args - Command line args to be passed
     * @name cy.callFirestore
     */
    Cypress.Commands.add('callFirestore', function (action, actionPath, data, opts) {
        if (opts === void 0) { opts = {}; }
        // If data is an object, create a copy to original object is not modified
        var dataToWrite = isObject(data) ? __assign({}, data) : data;
        // Add metadata to dataToWrite if specified by options
        if (isObject(data) && opts && opts.withMeta) {
            if (!dataToWrite.createdBy) {
                dataToWrite.createdBy = Cypress.env('TEST_UID');
            }
            if (!dataToWrite.createdAt) {
                dataToWrite.createdAt = new Date().toISOString();
            }
        }
        var firestoreCommand = buildFirestoreCommand(Cypress, action, actionPath, dataToWrite, opts);
        cy.log("Calling Firestore command:\n" + firestoreCommand);
        return cy.exec(firestoreCommand, { timeout: 100000 }).then(function (out) {
            var stdout = out.stdout, stderr = out.stderr;
            // Reject with Error if error in firestoreCommand call
            if (stderr) {
                cy.log("Error in Firestore Command:\n" + stderr);
                return Promise.reject(stderr);
            }
            // Parse result if using get action so that data can be verified
            if (action === 'get' &&
                (typeof stdout === 'string' || stdout instanceof String)) {
                try {
                    return JSON.parse(stdout instanceof String ? stdout.toString() : stdout);
                }
                catch (err) {
                    cy.log('Error parsing data from callFirestore response', out);
                    return Promise.reject(err);
                }
            }
            // Otherwise return unparsed output
            return stdout;
        });
    });
}
