import { __assign, __awaiter, __generator, __read } from "tslib";
import chalk from 'chalk';
import { writeFile } from 'fs';
import { pickBy, get, isUndefined } from 'lodash';
import { promisify } from 'util';
import { envVarBasedOnCIEnv, getServiceAccount, readJsonFile, getCypressConfigPath, withEnvPrefix, getEnvironmentSlug, } from './node-utils';
import { to } from './utils';
import { DEFAULT_TEST_ENV_FILE_NAME } from './constants';
import { FIREBASE_CONFIG_FILE_PATH, TEST_ENV_FILE_PATH } from './filePaths';
import * as logger from './logger';
var writeFilePromise = promisify(writeFile);
/* eslint-disable no-irregular-whitespace */
/**
 * Create test environment file (cypress.env.json). Uses admin.auth().createCustomToken
 * from firebase-admin authenticated with a Service Account which is loaded from environment
 * variables or config.json in test folder. Parameters which are added/copied:
 * - `TEST_UID`
 * - `FIREBASE_API_KEY`
 * - `FIREBASE_PROJECT_ID`
 * - `FIREBASE_AUTH_JWT`
 * @param envName - Environment name
 * @returns Promise which resolves with the contents of the test env file
 */
export default function createTestEnvFile(envName) {
    return __awaiter(this, void 0, void 0, function () {
        var varName, uid, errMsg, firebaserc, currentCypressEnvSettings, envSlug, FIREBASE_PROJECT_ID, serviceAccount, serviceAccountMissingParams, errMsg, cleanedProjectId, admin, appFromSA, developerClaims, defaultDeveloperClaims, _a, err, customToken, newCypressConfig;
        return __generator(this, function (_b) {
            switch (_b.label) {
                case 0:
                    varName = 'TEST_UID';
                    uid = envVarBasedOnCIEnv(varName, envName);
                    // Throw if UID is missing in environment
                    if (!uid) {
                        errMsg = chalk.cyan(varName) + " is missing from environment. Confirm that either " + chalk.cyan(withEnvPrefix(varName)) + " or " + chalk.cyan(varName) + " are set within environment variables, " + chalk.cyan(DEFAULT_TEST_ENV_FILE_NAME) + ", or " + chalk.cyan(getCypressConfigPath()) + ".";
                        throw new Error(errMsg);
                    }
                    firebaserc = readJsonFile(FIREBASE_CONFIG_FILE_PATH);
                    currentCypressEnvSettings = readJsonFile(TEST_ENV_FILE_PATH);
                    envSlug = envName || getEnvironmentSlug();
                    FIREBASE_PROJECT_ID = get(currentCypressEnvSettings, 'FIREBASE_PROJECT_ID') || // FIREBASE_PROJECT_ID already in cypress config
                        envVarBasedOnCIEnv('FIREBASE_PROJECT_ID', envSlug) || // FIREBASE_PROJECT_ID Environment variables
                        envVarBasedOnCIEnv('FIREBASE_PROJECT', envSlug) || // FIREBASE_PROJECT Environment variables
                        get(firebaserc, "ci.createConfig." + envSlug + ".firebase.projectId") || // CI createConfig projectId based on branch
                        get(firebaserc, "projects." + envSlug) || // project by branch name
                        get(firebaserc, 'projects.default', '');
                    logger.info("Generating custom auth token for Firebase project with projectId: " + chalk.cyan(FIREBASE_PROJECT_ID));
                    serviceAccount = getServiceAccount(envName);
                    serviceAccountMissingParams = pickBy(serviceAccount, isUndefined);
                    if (Object.keys(serviceAccountMissingParams).length > 0) {
                        errMsg = "Service Account is missing parameters: " + Object.keys(serviceAccountMissingParams).join(', ');
                        throw new Error(errMsg);
                    }
                    cleanedProjectId = FIREBASE_PROJECT_ID.replace('firebase-', '');
                    admin = require('firebase-admin');
                    appFromSA = admin.initializeApp({
                        credential: admin.credential.cert(serviceAccount),
                        databaseURL: "https://" + cleanedProjectId + ".firebaseio.com",
                    }, 'withServiceAccount');
                    developerClaims = envVarBasedOnCIEnv('DEVELOPER_CLAIMS', envName);
                    defaultDeveloperClaims = !!developerClaims && Object.keys(developerClaims).length > 0
                        ? developerClaims
                        : { isTesting: true };
                    return [4 /*yield*/, to(appFromSA.auth().createCustomToken(uid, defaultDeveloperClaims))];
                case 1:
                    _a = __read.apply(void 0, [_b.sent(), 2]), err = _a[0], customToken = _a[1];
                    // Handle errors generating custom token
                    if (err) {
                        logger.error("Custom token could not be generated for uid: " + chalk.cyan(uid), err.message || err);
                        throw err;
                    }
                    logger.success("Custom token generated successfully, writing to " + chalk.cyan(DEFAULT_TEST_ENV_FILE_NAME));
                    // Remove firebase app
                    appFromSA.delete();
                    newCypressConfig = __assign(__assign({}, currentCypressEnvSettings), { TEST_UID: uid, FIREBASE_PROJECT_ID: FIREBASE_PROJECT_ID, FIREBASE_API_KEY: envVarBasedOnCIEnv('FIREBASE_API_KEY', envName) ||
                            get(firebaserc, "ci.createConfig." + envSlug + ".firebase.apiKey", get(firebaserc, "ci.createConfig.master.firebase.apiKey")), FIREBASE_AUTH_JWT: customToken });
                    // Write config file to cypress.env.json
                    return [4 /*yield*/, writeFilePromise(TEST_ENV_FILE_PATH, JSON.stringify(newCypressConfig, null, 2))];
                case 2:
                    // Write config file to cypress.env.json
                    _b.sent();
                    logger.success(chalk.cyan(DEFAULT_TEST_ENV_FILE_NAME) + " updated successfully");
                    return [2 /*return*/, newCypressConfig];
            }
        });
    });
}
