import { __assign } from "tslib";
import { get } from 'lodash';
import { existsSync } from 'fs';
import { FIREBASE_CONFIG_FILE_NAME } from './constants';
import { readJsonFile } from './node-utils';
/**
 * Load .firebaserc file
 * @returns Contents of .firebaserc file parsed as JSON
 */
function loadFirebaseRc() {
    var rcFilePath = process.cwd() + "/" + FIREBASE_CONFIG_FILE_NAME;
    if (!existsSync(rcFilePath)) {
        throw new Error(FIREBASE_CONFIG_FILE_NAME + " file not found");
    }
    return readJsonFile(rcFilePath);
}
/**
 * Get environment name from cypress config or default to "local"
 * @param cypressConfig - Cypress config object
 * @returns Environment name from config
 */
function getEnvNameFromConfig(cypressConfig) {
    if (!cypressConfig.env || !cypressConfig.env.envName) {
        return 'local';
    }
    return cypressConfig.env.envName;
}
/**
 * Get Firebase project id using Cypress config and config
 * loaded from .firebaserc
 * @param config - Cypress config object
 * @returns Id of firbase project
 */
export function getFirebaseProjectIdFromConfig(config) {
    var projectIdFromConfig = get(config, 'env.firebaseProjectId');
    if (projectIdFromConfig) {
        return projectIdFromConfig;
    }
    var firbaseRcConfig = loadFirebaseRc();
    var envName = getEnvNameFromConfig(config);
    var projectsConfig = get(firbaseRcConfig, 'projects');
    var firebaseProjectId = projectsConfig[envName] || projectsConfig.master || projectsConfig.default;
    return firebaseProjectId;
}
/**
 * Load config for Cypress from .firebaserc.
 * @param cypressConfig - Existing Cypress config
 * @param settings - Settings
 * @returns Cypress config extended with FIREBASE_PROJECT_ID and baseUrl
 */
export default function extendWithFirebaseConfig(cypressConfig, settings) {
    if (settings === void 0) { settings = {}; }
    // Return original config if baseUrl is already set (so it is not runover)
    if (cypressConfig.baseUrl) {
        return cypressConfig;
    }
    var _a = settings, localBaseUrl = _a.localBaseUrl, _b = _a.localHostPort, localHostPort = _b === void 0 ? '3000' : _b;
    var envName = getEnvNameFromConfig(cypressConfig);
    var FIREBASE_PROJECT_ID = getFirebaseProjectIdFromConfig(cypressConfig);
    // Extend Firebase config with new config
    return __assign(__assign({}, cypressConfig), { FIREBASE_PROJECT_ID: FIREBASE_PROJECT_ID, baseUrl: envName === 'local'
            ? localBaseUrl || "http://localhost:" + localHostPort
            : "https://" + FIREBASE_PROJECT_ID + ".firebaseapp.com" });
}
