import { __read, __spread } from "tslib";
/* eslint-disable @typescript-eslint/camelcase */
import { get } from 'lodash';
import path from 'path';
import chalk from 'chalk';
import stream from 'stream';
import { spawn } from 'child_process';
import { existsSync, readFileSync } from 'fs';
import { TEST_CONFIG_FILE_PATH, TEST_ENV_FILE_PATH } from './filePaths';
import { DEFAULT_TEST_FOLDER_PATH, DEFAULT_CONFIG_FILE_NAME, } from './constants';
import { info, error, warn } from './logger';
var DEFAULT_BASE_PATH = process.cwd();
/**
 * Get settings from firebaserc file
 * @param filePath - Path for file
 * @returns Firebase settings object
 */
export function readJsonFile(filePath) {
    if (!existsSync(filePath)) {
        return {};
    }
    try {
        var fileBuffer = readFileSync(filePath, 'utf8');
        return JSON.parse(fileBuffer.toString());
    }
    catch (err) {
        error("Unable to parse " + chalk.cyan(filePath.replace(DEFAULT_BASE_PATH, '')) + " - JSON is most likely not valid");
        return {};
    }
}
/**
 * Get branch name from GITHUB_REF environment variable which is
 * available in Github Actions environment.
 * @returns Branch name if environment variable exists
 */
function branchNameForGithubAction() {
    var _a = process.env, GITHUB_HEAD_REF = _a.GITHUB_HEAD_REF, GITHUB_REF = _a.GITHUB_REF;
    // GITHUB_HEAD_REF for pull requests
    if (GITHUB_HEAD_REF) {
        return GITHUB_HEAD_REF;
    }
    // GITHUB_REF for commits (i.e. refs/heads/master)
    if (GITHUB_REF) {
        return GITHUB_REF.replace('refs/heads/', ''); // remove prefix if it exists
    }
}
/**
 * Get environment slug
 * @returns Environment slug
 */
export function getEnvironmentSlug() {
    return (branchNameForGithubAction() ||
        process.env.CI_ENVIRONMENT_SLUG || // Gitlab-CI "environment" param
        process.env.CI_COMMIT_REF_SLUG || // Gitlab-CI
        'master');
}
/**
 * Get prefix for current environment based on environment vars available
 * within CI. Falls back to staging (i.e. STAGE)
 * @param envName - Environment option
 * @returns Environment prefix string
 */
export function getEnvPrefix(envName) {
    var envSlug = envName || getEnvironmentSlug();
    return envSlug.toUpperCase() + "_";
}
/**
 * Create a variable name string with environment prefix (i.e. STAGE_SERVICE_ACCOUNT)
 * @param varNameRoot - Root of environment variable name
 * @param envName - Environment option
 * @returns Environment var name with prefix
 */
export function withEnvPrefix(varNameRoot, envName) {
    var envPrefix = getEnvPrefix(envName);
    return "" + envPrefix + varNameRoot;
}
/**
 * Get path to local service account
 * @param envName - Environment option
 * @returns Path to service account
 */
function getServiceAccountPath(envName) {
    var withSuffix = path.join(DEFAULT_BASE_PATH, "serviceAccount-" + (envName || '') + ".json");
    if (existsSync(withSuffix)) {
        return withSuffix;
    }
    return path.join(DEFAULT_BASE_PATH, 'serviceAccount.json');
}
/**
 * Get cypress folder path from cypress.json config file or fallback to
 * default folder path ('cypress')
 * @returns Path of folder containing cypress folders like "integration"
 */
function getCypressFolderPath() {
    var cypressConfig = readJsonFile(TEST_CONFIG_FILE_PATH); // eslint-disable-line no-use-before-define
    var integrationTestsFolderPath = get(cypressConfig, 'integrationFolder');
    return integrationTestsFolderPath
        ? integrationTestsFolderPath
            .split('/')
            .slice(0, -1) // Drop last item (equivalent to dropRight)
            .join('/')
        : DEFAULT_TEST_FOLDER_PATH;
}
/**
 * Get path to cypress config file
 * @returns Path to cypress config file
 */
export function getCypressConfigPath() {
    var cypressFolderPath = getCypressFolderPath();
    var cypressConfigFilePath = path.join(cypressFolderPath, DEFAULT_CONFIG_FILE_NAME);
    return cypressConfigFilePath;
}
/**
 * Get environment variable based on the current CI environment
 * @param varNameRoot - variable name without the environment prefix
 * @param envName - Environment option
 * @returns Value of the environment variable
 * @example
 * envVarBasedOnCIEnv('FIREBASE_PROJECT_ID')
 * // => 'fireadmin-stage' (value of 'STAGE_FIREBASE_PROJECT_ID' environment var)
 */
export function envVarBasedOnCIEnv(varNameRoot, envName) {
    var combined = withEnvPrefix(varNameRoot, envName);
    var localConfigFilePath = getCypressConfigPath();
    // Config file used for environment (local, containers) from main test path ({integrationFolder}/config.json)
    if (existsSync(localConfigFilePath)) {
        var localConfigObj = readJsonFile(localConfigFilePath);
        var valueFromLocalConfig = localConfigObj[combined] || localConfigObj[varNameRoot];
        if (valueFromLocalConfig) {
            return valueFromLocalConfig;
        }
    }
    // Config file used for environment from main cypress environment file (cypress.env.json)
    if (existsSync(TEST_ENV_FILE_PATH)) {
        var configObj = readJsonFile(TEST_ENV_FILE_PATH);
        var valueFromCypressEnv = configObj[combined] || configObj[varNameRoot];
        if (valueFromCypressEnv) {
            return valueFromCypressEnv;
        }
    }
    // CI Environment (environment variables loaded directly)
    return process.env[combined] || process.env[varNameRoot];
}
/**
 * Get parsed value of environment variable. Useful for environment variables
 * which have characters that need to be escaped.
 * @param varNameRoot - variable name without the environment prefix
 * @param envName - Environment option
 * @returns Value of the environment variable
 * @example
 * getParsedEnvVar('FIREBASE_PRIVATE_KEY_ID')
 * // => 'fireadmin-stage' (parsed value of 'STAGE_FIREBASE_PRIVATE_KEY_ID' environment var)
 */
function getParsedEnvVar(varNameRoot, envName) {
    var val = envVarBasedOnCIEnv(varNameRoot, envName);
    var combinedVar = withEnvPrefix(varNameRoot, envName);
    if (!val) {
        error(chalk.cyan(combinedVar) + " not found, make sure it is set within environment variables.");
    }
    try {
        if (typeof val === 'string') {
            return JSON.parse(val);
        }
        return val;
    }
    catch (err) {
        error("Error parsing " + combinedVar);
        return val;
    }
}
/**
 * Get service account from either local file or environment variables
 * @param envSlug - Environment option
 * @returns Service account object
 */
export function getServiceAccount(envSlug) {
    var serviceAccountPath = getServiceAccountPath(envSlug);
    // Check for local service account file (Local dev)
    if (existsSync(serviceAccountPath)) {
        return readJsonFile(serviceAccountPath); // eslint-disable-line global-require, import/no-dynamic-require
    }
    info("Service account does not exist at path: \"" + chalk.cyan(serviceAccountPath.replace(DEFAULT_BASE_PATH + "/", '')) + "\" falling back to environment variables...");
    // Use environment variables (CI)
    var serviceAccountEnvVar = envVarBasedOnCIEnv('SERVICE_ACCOUNT', envSlug);
    if (serviceAccountEnvVar) {
        if (typeof serviceAccountEnvVar === 'string') {
            try {
                return JSON.parse(serviceAccountEnvVar);
            }
            catch (err) {
                warn("Issue parsing " + chalk.cyan('SERVICE_ACCOUNT') + " environment variable from string to object, returning string");
            }
        }
        return serviceAccountEnvVar;
    }
    info("Service account does not exist as a single environment variable within " + chalk.cyan('SERVICE_ACCOUNT') + " or " + chalk.cyan(withEnvPrefix('SERVICE_ACCOUNT')) + ", checking separate environment variables...");
    var clientId = envVarBasedOnCIEnv('FIREBASE_CLIENT_ID', envSlug);
    if (clientId) {
        warn(chalk.cyan('FIREBASE_CLIENT_ID') + " will override " + chalk.cyan('FIREBASE_TOKEN') + " for auth when calling firebase-tools - this may cause unexepected behavior");
    }
    return {
        type: 'service_account',
        project_id: envVarBasedOnCIEnv('FIREBASE_PROJECT_ID', envSlug),
        private_key_id: envVarBasedOnCIEnv('FIREBASE_PRIVATE_KEY_ID', envSlug),
        private_key: getParsedEnvVar('FIREBASE_PRIVATE_KEY', envSlug),
        client_email: envVarBasedOnCIEnv('FIREBASE_CLIENT_EMAIL', envSlug),
        client_id: clientId,
        auth_uri: 'https://accounts.google.com/o/oauth2/auth',
        token_uri: 'https://accounts.google.com/o/oauth2/token',
        auth_provider_x509_cert_url: 'https://www.googleapis.com/oauth2/v1/certs',
        client_x509_cert_url: envVarBasedOnCIEnv('FIREBASE_CERT_URL', envSlug),
    };
}
/**
 * Run a bash command using spawn pipeing the results to the main process
 * @param runOptions - Options for command run
 * @param runOptions.command - Command to be executed
 * @param runOptions.args - Command arguments
 * @returns Resolves with results of running the command
 * @private
 */
export function runCommand(runOptions) {
    var command = runOptions.command, args = runOptions.args, _a = runOptions.pipeOutput, pipeOutput = _a === void 0 ? true : _a;
    return new Promise(function (resolve, reject) {
        var child = spawn(command, args);
        var output;
        var error;
        var customStream = new stream.Writable();
        var customErrorStream = new stream.Writable();
        /* eslint-disable no-underscore-dangle */
        customStream._write = function (data) {
            var _a;
            var argv = [];
            for (var _i = 1; _i < arguments.length; _i++) {
                argv[_i - 1] = arguments[_i];
            }
            output += data;
            if (pipeOutput) {
                (_a = process.stdout)._write.apply(_a, __spread([data], argv));
            }
        };
        customErrorStream._write = function (data) {
            var _a;
            var argv = [];
            for (var _i = 1; _i < arguments.length; _i++) {
                argv[_i - 1] = arguments[_i];
            }
            error += data;
            if (pipeOutput) {
                (_a = process.stderr)._write.apply(_a, __spread([data], argv));
            }
        };
        /* eslint-enable no-underscore-dangle */
        // Pipe errors and console output to main process
        child.stdout.pipe(customStream);
        child.stderr.pipe(customErrorStream);
        // When child exits resolve or reject based on code
        child.on('exit', function (code) {
            if (code !== 0) {
                // Resolve for npm warnings
                if (output && output.includes('npm WARN')) {
                    return resolve(output);
                }
                reject(error || output);
            }
            else {
                // Remove leading undefined from response
                resolve(output && output.indexOf('undefined') === 0
                    ? output.replace('undefined', '')
                    : output);
            }
        });
    });
}
process.env.FORCE_COLOR = 'true';
