"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var lodash_1 = require("lodash");
var utils_1 = require("./utils");
var constants_1 = require("./constants");
var ARGS_MAP = {
    orderByChild: '--order-by',
    orderByKey: '--order-by-key',
    orderByValue: '--order-by-value',
    limitToFirst: '--limit-to-first',
    limitToLast: '--limit-to-last',
    startAt: '--start-at',
    endAt: '--end-at',
    equalTo: '--equal-to',
    instance: '--instance',
};
/**
 * Generate RTDB args from command options
 * @param Cypress - Cypress object
 * @param args - Extra arguments to be passed with command
 * @param [options={}] - Options object
 * @returns A list of args for get command
 */
function generageRtdbGetArgs(Cypress, args, options) {
    var getDataArgsWithDefaults = utils_1.addDefaultArgs(Cypress, args, {
        disableYes: true,
    });
    if (options.orderByChild && !options.limitToLast && !options.limitToLast) {
        getDataArgsWithDefaults.push(ARGS_MAP.orderByChild);
        getDataArgsWithDefaults.push(options.orderByChild);
    }
    if (options.orderByKey && !options.limitToLast) {
        getDataArgsWithDefaults.push(ARGS_MAP.orderByKey);
    }
    if (options.orderByValue) {
        getDataArgsWithDefaults.push(ARGS_MAP.orderByValue);
    }
    if (options.startAt) {
        getDataArgsWithDefaults.push(ARGS_MAP.startAt);
        getDataArgsWithDefaults.push(options.startAt);
    }
    if (options.endAt) {
        getDataArgsWithDefaults.push(ARGS_MAP.endAt);
        getDataArgsWithDefaults.push(options.endAt);
    }
    if (options.equalTo) {
        getDataArgsWithDefaults.push(ARGS_MAP.equalTo);
        getDataArgsWithDefaults.push(options.equalTo);
    }
    if (options.instance) {
        getDataArgsWithDefaults.push(ARGS_MAP.instance);
        getDataArgsWithDefaults.push(options.instance);
    }
    if (options.limitToLast) {
        var lastCount = lodash_1.isBoolean(options.limitToLast) ? 1 : options.limitToLast;
        if (!options.orderByChild) {
            getDataArgsWithDefaults.push("--order-by-key --limit-to-last " + lastCount);
        }
        else {
            getDataArgsWithDefaults.push("--order-by " + options.orderByChild + " --limit-to-last " + lastCount);
        }
    }
    if (options.limitToFirst) {
        var lastCount = lodash_1.isBoolean(options.limitToFirst)
            ? 1
            : options.limitToFirst;
        if (!options.orderByChild) {
            getDataArgsWithDefaults.push("--order-by-key --limit-to-first " + lastCount);
        }
        else {
            getDataArgsWithDefaults.push("--order-by " + options.orderByChild + " --limit-to-first " + lastCount);
        }
    }
    return getDataArgsWithDefaults;
}
/**
 * Build Command to run Real Time Database action. All commands call
 * firebase-tools directly, so FIREBASE_TOKEN must exist in environment.
 * @param Cypress - Cypress object
 * @param action - action to run on Firstore (i.e. "add", "delete")
 * @param actionPath - Firestore path where action should be run
 * @param fixturePath - Path to fixture. If object is passed,
 * it is used as options.
 * @param [opts={}] - Options object
 * @param opts.args - Extra arguments to be passed with command
 * @returns Command string to be used with cy.exec
 */
function buildRtdbCommand(Cypress, action, actionPath, fixturePath, opts) {
    var options = lodash_1.isObject(fixturePath)
        ? fixturePath
        : opts || {};
    var _a = options.args, args = _a === void 0 ? [] : _a;
    var argsWithDefaults = utils_1.addDefaultArgs(Cypress, args);
    var argsStr = utils_1.getArgsString(argsWithDefaults);
    // Add preceding slash if it doesn't already exist (required by firebase-tools)
    var cleanActionPath = actionPath.startsWith('/')
        ? actionPath
        : "/" + actionPath;
    switch (action) {
        case 'remove':
            return constants_1.FIREBASE_TOOLS_BASE_COMMAND + " database:" + action + " " + cleanActionPath + argsStr;
        case 'delete':
            return constants_1.FIREBASE_TOOLS_BASE_COMMAND + " database:remove " + cleanActionPath + argsStr;
        case 'get': {
            var getDataArgsWithDefaults = generageRtdbGetArgs(Cypress, args, options);
            var getDataArgsStr = utils_1.getArgsString(getDataArgsWithDefaults);
            return constants_1.FIREBASE_TOOLS_BASE_COMMAND + " database:" + action + " " + cleanActionPath + getDataArgsStr;
        }
        default: {
            var commandString = constants_1.FIREBASE_TOOLS_BASE_COMMAND + " database:" + action + " " + cleanActionPath;
            if (!lodash_1.isObject(fixturePath)) {
                return commandString + " " + fixturePath + argsStr;
            }
            return commandString + " -d '" + JSON.stringify(fixturePath) + "'" + argsStr;
        }
    }
}
exports.default = buildRtdbCommand;
