"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var tslib_1 = require("tslib");
var chalk_1 = tslib_1.__importDefault(require("chalk"));
var fs_1 = require("fs");
var lodash_1 = require("lodash");
var util_1 = require("util");
var node_utils_1 = require("./node-utils");
var utils_1 = require("./utils");
var constants_1 = require("./constants");
var filePaths_1 = require("./filePaths");
var logger = tslib_1.__importStar(require("./logger"));
var writeFilePromise = util_1.promisify(fs_1.writeFile);
/* eslint-disable no-irregular-whitespace */
/**
 * Create test environment file (cypress.env.json). Uses admin.auth().createCustomToken
 * from firebase-admin authenticated with a Service Account which is loaded from environment
 * variables or config.json in test folder. Parameters which are added/copied:
 * - `TEST_UID`
 * - `FIREBASE_API_KEY`
 * - `FIREBASE_PROJECT_ID`
 * - `FIREBASE_AUTH_JWT`
 * @param envName - Environment name
 * @returns Promise which resolves with the contents of the test env file
 */
function createTestEnvFile(envName) {
    return tslib_1.__awaiter(this, void 0, void 0, function () {
        var varName, uid, errMsg, firebaserc, currentCypressEnvSettings, envSlug, FIREBASE_PROJECT_ID, serviceAccount, serviceAccountMissingParams, errMsg, cleanedProjectId, admin, appFromSA, developerClaims, defaultDeveloperClaims, _a, err, customToken, newCypressConfig;
        return tslib_1.__generator(this, function (_b) {
            switch (_b.label) {
                case 0:
                    varName = 'TEST_UID';
                    uid = node_utils_1.envVarBasedOnCIEnv(varName, envName);
                    // Throw if UID is missing in environment
                    if (!uid) {
                        errMsg = chalk_1.default.cyan(varName) + " is missing from environment. Confirm that either " + chalk_1.default.cyan(node_utils_1.withEnvPrefix(varName)) + " or " + chalk_1.default.cyan(varName) + " are set within environment variables, " + chalk_1.default.cyan(constants_1.DEFAULT_TEST_ENV_FILE_NAME) + ", or " + chalk_1.default.cyan(node_utils_1.getCypressConfigPath()) + ".";
                        throw new Error(errMsg);
                    }
                    firebaserc = node_utils_1.readJsonFile(filePaths_1.FIREBASE_CONFIG_FILE_PATH);
                    currentCypressEnvSettings = node_utils_1.readJsonFile(filePaths_1.TEST_ENV_FILE_PATH);
                    envSlug = envName || node_utils_1.getEnvironmentSlug();
                    FIREBASE_PROJECT_ID = lodash_1.get(currentCypressEnvSettings, 'FIREBASE_PROJECT_ID') || // FIREBASE_PROJECT_ID already in cypress config
                        node_utils_1.envVarBasedOnCIEnv('FIREBASE_PROJECT_ID', envSlug) || // FIREBASE_PROJECT_ID Environment variables
                        node_utils_1.envVarBasedOnCIEnv('FIREBASE_PROJECT', envSlug) || // FIREBASE_PROJECT Environment variables
                        lodash_1.get(firebaserc, "ci.createConfig." + envSlug + ".firebase.projectId") || // CI createConfig projectId based on branch
                        lodash_1.get(firebaserc, "projects." + envSlug) || // project by branch name
                        lodash_1.get(firebaserc, 'projects.default', '');
                    logger.info("Generating custom auth token for Firebase project with projectId: " + chalk_1.default.cyan(FIREBASE_PROJECT_ID));
                    serviceAccount = node_utils_1.getServiceAccount(envName);
                    serviceAccountMissingParams = lodash_1.pickBy(serviceAccount, lodash_1.isUndefined);
                    if (Object.keys(serviceAccountMissingParams).length > 0) {
                        errMsg = "Service Account is missing parameters: " + Object.keys(serviceAccountMissingParams).join(', ');
                        throw new Error(errMsg);
                    }
                    cleanedProjectId = FIREBASE_PROJECT_ID.replace('firebase-', '');
                    admin = require('firebase-admin');
                    appFromSA = admin.initializeApp({
                        credential: admin.credential.cert(serviceAccount),
                        databaseURL: "https://" + cleanedProjectId + ".firebaseio.com",
                    }, 'withServiceAccount');
                    developerClaims = node_utils_1.envVarBasedOnCIEnv('DEVELOPER_CLAIMS', envName);
                    defaultDeveloperClaims = !!developerClaims && Object.keys(developerClaims).length > 0
                        ? developerClaims
                        : { isTesting: true };
                    return [4 /*yield*/, utils_1.to(appFromSA.auth().createCustomToken(uid, defaultDeveloperClaims))];
                case 1:
                    _a = tslib_1.__read.apply(void 0, [_b.sent(), 2]), err = _a[0], customToken = _a[1];
                    // Handle errors generating custom token
                    if (err) {
                        logger.error("Custom token could not be generated for uid: " + chalk_1.default.cyan(uid), err.message || err);
                        throw err;
                    }
                    logger.success("Custom token generated successfully, writing to " + chalk_1.default.cyan(constants_1.DEFAULT_TEST_ENV_FILE_NAME));
                    // Remove firebase app
                    appFromSA.delete();
                    newCypressConfig = tslib_1.__assign(tslib_1.__assign({}, currentCypressEnvSettings), { TEST_UID: uid, FIREBASE_PROJECT_ID: FIREBASE_PROJECT_ID, FIREBASE_API_KEY: node_utils_1.envVarBasedOnCIEnv('FIREBASE_API_KEY', envName) ||
                            lodash_1.get(firebaserc, "ci.createConfig." + envSlug + ".firebase.apiKey", lodash_1.get(firebaserc, "ci.createConfig.master.firebase.apiKey")), FIREBASE_AUTH_JWT: customToken });
                    // Write config file to cypress.env.json
                    return [4 /*yield*/, writeFilePromise(filePaths_1.TEST_ENV_FILE_PATH, JSON.stringify(newCypressConfig, null, 2))];
                case 2:
                    // Write config file to cypress.env.json
                    _b.sent();
                    logger.success(chalk_1.default.cyan(constants_1.DEFAULT_TEST_ENV_FILE_NAME) + " updated successfully");
                    return [2 /*return*/, newCypressConfig];
            }
        });
    });
}
exports.default = createTestEnvFile;
