"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var tslib_1 = require("tslib");
var lodash_1 = require("lodash");
var fs_1 = require("fs");
var constants_1 = require("./constants");
var node_utils_1 = require("./node-utils");
/**
 * Load .firebaserc file
 * @returns Contents of .firebaserc file parsed as JSON
 */
function loadFirebaseRc() {
    var rcFilePath = process.cwd() + "/" + constants_1.FIREBASE_CONFIG_FILE_NAME;
    if (!fs_1.existsSync(rcFilePath)) {
        throw new Error(constants_1.FIREBASE_CONFIG_FILE_NAME + " file not found");
    }
    return node_utils_1.readJsonFile(rcFilePath);
}
/**
 * Get environment name from cypress config or default to "local"
 * @param cypressConfig - Cypress config object
 * @returns Environment name from config
 */
function getEnvNameFromConfig(cypressConfig) {
    if (!cypressConfig.env || !cypressConfig.env.envName) {
        return 'local';
    }
    return cypressConfig.env.envName;
}
/**
 * Get Firebase project id using Cypress config and config
 * loaded from .firebaserc
 * @param config - Cypress config object
 * @returns Id of firbase project
 */
function getFirebaseProjectIdFromConfig(config) {
    var projectIdFromConfig = lodash_1.get(config, 'env.firebaseProjectId');
    if (projectIdFromConfig) {
        return projectIdFromConfig;
    }
    var firbaseRcConfig = loadFirebaseRc();
    var envName = getEnvNameFromConfig(config);
    var projectsConfig = lodash_1.get(firbaseRcConfig, 'projects');
    var firebaseProjectId = projectsConfig[envName] || projectsConfig.master || projectsConfig.default;
    return firebaseProjectId;
}
exports.getFirebaseProjectIdFromConfig = getFirebaseProjectIdFromConfig;
/**
 * Load config for Cypress from .firebaserc.
 * @param cypressConfig - Existing Cypress config
 * @param settings - Settings
 * @returns Cypress config extended with FIREBASE_PROJECT_ID and baseUrl
 */
function extendWithFirebaseConfig(cypressConfig, settings) {
    if (settings === void 0) { settings = {}; }
    // Return original config if baseUrl is already set (so it is not runover)
    if (cypressConfig.baseUrl) {
        return cypressConfig;
    }
    var _a = settings, localBaseUrl = _a.localBaseUrl, _b = _a.localHostPort, localHostPort = _b === void 0 ? '3000' : _b;
    var envName = getEnvNameFromConfig(cypressConfig);
    var FIREBASE_PROJECT_ID = getFirebaseProjectIdFromConfig(cypressConfig);
    // Extend Firebase config with new config
    return tslib_1.__assign(tslib_1.__assign({}, cypressConfig), { FIREBASE_PROJECT_ID: FIREBASE_PROJECT_ID, baseUrl: envName === 'local'
            ? localBaseUrl || "http://localhost:" + localHostPort
            : "https://" + FIREBASE_PROJECT_ID + ".firebaseapp.com" });
}
exports.default = extendWithFirebaseConfig;
