"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var tslib_1 = require("tslib");
var constants_1 = require("./constants");
/**
 * Async await wrapper for easy error handling
 * @param promise - Promise to wrap responses of in array
 * @param errorExt - Extension for error
 * @returns Resolves and rejects with an array
 */
function to(promise, errorExt) {
    return promise
        .then(function (data) { return [null, data]; })
        .catch(function (err) {
        if (errorExt) {
            Object.assign(err, errorExt);
        }
        return [err, undefined];
    });
}
exports.to = to;
/**
 * Create command arguments string from an array of arguments by joining them
 * with a space including a leading space. If no args provided, empty string
 * is returned
 * @param args - Command arguments to convert into a string
 * @returns Arguments section of command string
 */
function getArgsString(args) {
    return args && args.length ? " " + args.join(' ') : '';
}
exports.getArgsString = getArgsString;
/**
 * Add default Firebase arguments to arguments array.
 * @param Cypress - Cypress object
 * @param args - arguments array
 * @param [opts={}] - Options object
 * @param opts.token - Firebase CI token to pass as the token argument
 * @param [opts.disableYes=false] - Whether or not to disable the yes argument
 * @returns Default args list
 */
function addDefaultArgs(Cypress, args, opts) {
    var _a = opts || {}, _b = _a.disableYes, disableYes = _b === void 0 ? false : _b, token = _a.token, withMeta = _a.withMeta;
    var newArgs = tslib_1.__spread(args);
    // TODO: Load this in a way that understands environment. Currently this will
    // go to the first project id that is defined, not which one should be used
    // for the specified environment
    var projectId = Cypress.env('firebaseProjectId') ||
        Cypress.env('FIREBASE_PROJECT_ID') ||
        Cypress.env('STAGE_FIREBASE_PROJECT_ID');
    // Include project id command so command runs on the current project
    if (projectId && !newArgs.includes('-P') && !newArgs.includes(projectId)) {
        newArgs.push('-P');
        newArgs.push(projectId);
    }
    var tokenFromEnv = Cypress.env('FIREBASE_TOKEN');
    // Include token if it exists in environment
    if (!newArgs.includes('--token') && (token || tokenFromEnv)) {
        newArgs.push('--token');
        newArgs.push(token || tokenFromEnv);
    }
    // Add Firebase's automatic approval argument if it is not already in newArgs
    if (!disableYes && !newArgs.includes(constants_1.FIREBASE_TOOLS_YES_ARGUMENT)) {
        newArgs.push(constants_1.FIREBASE_TOOLS_YES_ARGUMENT);
    }
    if (withMeta) {
        // Add -m to argsWithDefaults string (meta) if withmeta option is true
        newArgs.push('-m');
    }
    return newArgs;
}
exports.addDefaultArgs = addDefaultArgs;
